import traceback

from ...api.classes import JobJSON
from ...session import StaticSession


def run_job(job: JobJSON, code: str):
    session = StaticSession("jobs", job.identifier)

    try:
        namespace: dict = {}
        exec(code, namespace, namespace)
    except:
        traceback.print_exc()



from ..widget_base import Input
from typing import Union, List, Dict


class DropdownInput(Input):
    type = "dropdown-input"

    def __init__(
        self, key: str, label: str, options: Union[List[str], List[Dict]], **kwargs
    ):
        super().__init__(key)
        self.label = label
        self.options = options
        self.initial_value = kwargs.get("initial_value", None)
        self.required = kwargs.get("required", True)
        self.hint = kwargs.get("hint", None)
        self.multiple = kwargs.get("multiple", False)
        self.placeholder = kwargs.get("placeholder", "Select your preference")
        self.columns = kwargs.get("columns", 1)
        self.full_width = kwargs.get("full_width", False)
        self.disabled = kwargs.get("disabled", False)

    def json(self, **kwargs):
        return {
            "type": self.type,
            "key": self.key,
            "label": self.label,
            "options": self.options,
            "hint": self.hint,
            "multiple": self.multiple,
            "placeholder": self.placeholder,
            "initialValue": self.initial_value,
            "required": self.required,
            "columns": self.columns,
            "fullWidth": self.full_width,
            "disabled": self.disabled,
        }

    def convert_answer(self, answer: str) -> str:
        """
        Returns:
            str: The value selected by the user
        """
        return answer



import inspect, typing
from types import TracebackType
from dataclasses import dataclass


Frames = typing.Union[typing.List[inspect.FrameInfo], None]


def representations(locals):
    result = {}
    for key, value in list(locals.items()):
        try:
            result[key] = repr(value)
        except:
            pass
    return result


def traceback_to_infos(
    traceback: typing.Union[TracebackType, None]
) -> typing.List[inspect.FrameInfo]:
    return inspect.getouterframes(traceback.tb_frame) if traceback else []


def make_debug_data(frames: Frames):
    if not frames:
        return {}
    return {
        "debug": {
            "stack": [
                {
                    "filename": info.filename,
                    "lineno": info.lineno,
                    "name": info.function,
                    "locals": representations(info.frame.f_locals),
                }
                for info in (frames)
            ]
        }
    }


@dataclass
class CloseDTO:
    exit_code: typing.Union[int, None] = None
    exception: typing.Union[str, None] = None
    frames: Frames = None



import pandas as pd
import plotly.graph_objs as go
from abstra.widgets.library import *

example_instances = [
    TextInput("text", "What is your name?"),
    TagInput(
        "tags",
        "What are your interests?",
        initial_value=["python", "javascript", "c++"],
    ),
    DateInput("date", "When is your birthday?"),
    FileInput("file", "Upload a file"),
    ImageInput("file", "Upload a image file"),
    VideoInput("file", "Upload a video file"),
    MultipleChoiceInput(
        "multiple-choice",
        "What is your favorite color?",
        options=["Red", "Blue", "Green"],
    ),
    DropdownInput(
        "dropdown", "What is your favorite color?", options=["Red", "Blue", "Green"]
    ),
    TextareaInput("textarea", "What is your favorite color?"),
    NumberInput("number", "What is your favorite number?"),
    EmailInput("email", "What is your email address?"),
    PhoneInput("phone", "What is your phone number?"),
    CardsInput(
        "card",
        "What is your favorite color?",
        options=[
            {
                "title": "red",
                "image": "https://via.placeholder.com/150/FF0000/808080",
                "description": "red",
            },
            {
                "title": "blue",
                "image": "https://via.placeholder.com/150/0000FF/808080",
                "description": "blue",
            },
            {
                "title": "green",
                "image": "https://via.placeholder.com/150/00FF00/808080",
                "description": "green",
            },
        ],
    ),
    TextOutput("Hello, world!"),
    MarkdownOutput("## This is a h2 tag"),
    ImageOutput("https://i.imgur.com/XyqQZ.jpg", subtitle="A cute cat"),
    LinkOutput("https://www.google.com", link_text="Google"),
    FileOutput("https://www.google.com", download_text="Google"),
    HtmlOutput("<h1>Hello, world!</h1>"),
    PandasOutput(pd.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]})),
    PlotlyOutput(go.Figure(data=[go.Scatter(x=[1, 2, 3], y=[2, 1, 2])])),
    IframeOutput("<h1>Hello, world!</h1>", width="100%", height="100%", column=1),
    PandasRowSelectionInput(
        "dataframe", pd.DataFrame(data={"col1": [1, 2], "col2": [3, 4]})
    ),
    TimeInput("time", "What's the meeting time?"),
    PasswordInput("password", "Insert your password"),
    CodeInput("code", "Type your code", language="python"),
    NpsInput(
        "nps", "How likely?", min=1, max=100, min_hint="No way!", max_hint="Hell yeah!"
    ),
    CurrencyInput("currency", "How much?", currency="BRL"),
    ProgressOutput(50, 100),
    NumberSliderInput("slider", "How much?", min=0, max=100, step=1),
    CheckboxInput("checkbox", "Do you agree?"),
    CnpjInput("cnpj", "What is your CNPJ?"),
    CpfInput("cpf", "What is your CPF?"),
    ToggleInput("toggle", "Do you agree?"),
    ChecklistInput(
        "checklist",
        "What are your interests?",
        options=["python", "javascript", "c++"],
    ),
    KanbanBoardInput(
        "kanban",
        label="What are your interests?",
    ),
    AnswerSheetInput(
        "answer-sheet",
        "What are your interests?",
        options=["python", "javascript", "c++"],
        number_of_questions=3,
    ),
    LatexOutput("$$\\frac{1}{2}$$"),
    RatingInput("rating", "How much?", max=10),
    ClickInput("click", "Click me!"),
    CustomInput(
        "custom",
        "<button id='date-btn'>Get current date</button>",
        label="Custom Widget",
        js="console.log('test')",
        css="button { border: none; }",
        html_head="",
    ),
    RichTextInput("rich-text", "Write your blog post here"),
]



import traceback, flask_sock

from ...session import LiveSession
from ...api.classes import FormJSON
from ...contract import forms_contract
from .message_handler import MessageBroker

from abstra.forms.debug_utils import CloseDTO, traceback_to_infos


def run_form(conn: flask_sock.Server, form: FormJSON, code: str):
    session = LiveSession(conn, "forms", form.path)
    namespace: dict = {}
    close_dto = CloseDTO(exit_code=0)
    broker = MessageBroker(session)
    session.context.update(broker=broker)
    session.send(forms_contract.SessionIdMessage(session.id))

    try:
        exec(code, namespace, namespace)
    except Exception as e:
        traceback.print_exc()
        close_dto.frames = traceback_to_infos(e.__traceback__)
        close_dto.exception = e.__str__()
        close_dto.exit_code = 1
    finally:
        broker.close(close_dto)



import tempfile
import unittest
from uuid import uuid4 as uuid
from pathlib import Path

from abstra_server.api import API

from .fixtures import init_dir, clear_dir


class TestHooks(unittest.TestCase):
    def setUp(self) -> None:
        self.root = init_dir()

    def tearDown(self) -> None:
        clear_dir(self.root)

    def test_api_list(self):
        api = API(root=self.root)

        self.assertEqual(len(api.get_hooks()), 0)

        api.create_hook()

        self.assertEqual(len(api.get_hooks()), 1)


if __name__ == "__main__":
    unittest.main()



import sys
from abstra import dashes as abstra_dashes
from abstra import hooks as abstra_hooks
from abstra import forms as abstra_forms

from abstra.forms.overloads import overload_to_widgets

from . import flows
from .contract import dashes_contract
from .session import LiveSession, StaticSession, Execution
from .runtimes.forms.message_handler import MessageBroker


def get_live_session_throwable() -> LiveSession:
    session = LiveSession.get_session()
    if not session:
        raise Exception("No session found")
    return session


def get_static_session_throwable() -> StaticSession:
    session = StaticSession.get_session()
    if not session:
        raise Exception("No session found")
    return session


def __overload_abstra_dashes_sdk():
    def get_user():
        session = get_live_session_throwable()
        return flows.get_user(session)

    def redirect(url, query_params=None):
        session = get_live_session_throwable()
        query_params = query_params or session.context.get("query_params", {})
        session.send(dashes_contract.RedirectMessage(url, query_params))

    def get_query_params():
        session = get_live_session_throwable()
        return session.context.get("query_params", {})

    def alert(message, severity="info"):
        session = get_live_session_throwable()

        if severity not in ["info", "warn", "error", "success"]:
            severity = "info"

        session.send(dashes_contract.AlertMessage(message, severity))

    def execute_js(code, context={}):
        session = get_live_session_throwable()
        session.send(dashes_contract.ExecuteJSRequestMessage(code, context))

        while True:
            type, data = session.recv()
            if type != "execute-js:response":
                continue

            return data.get("value")

    abstra_dashes.get_user = get_user
    abstra_dashes.redirect = redirect
    abstra_dashes.get_query_params = get_query_params
    abstra_dashes.alert = alert
    abstra_dashes.execute_js = execute_js


def __overload_abstra_hooks_sdk():
    def get_raw_request(_file=None):
        session = get_static_session_throwable()
        raw, args, headers = session.context.get("request")
        return raw, args, headers

    def send_response(body="", status_code=200, headers={}, local_file=None):
        session = get_static_session_throwable()
        session.context["response"] = (
            body,
            status_code,
            headers,
        )

    abstra_hooks.get_raw_request = get_raw_request
    abstra_hooks.send_response = send_response


def __overload_abstra_forms_sdk():
    def get_connection() -> MessageBroker:
        session = get_live_session_throwable()
        broker = session.context.get("broker")
        if not broker:
            raise Exception("No broker found")

        return broker

    def get_user():
        session = get_live_session_throwable()
        return flows.get_user(session)

    abstra_forms.socket.get_connection = get_connection
    abstra_forms.auth.get_user = get_user
    abstra_forms.get_user = get_user


def __overload_stdio():
    def writeWraper(type, write, text):
        try:
            write(text)
            execution = Execution.get_execution()
            if execution:
                execution.stdio(type, text)
        finally:
            return len(text)

    stdout_write = sys.stdout.write
    stderr_write = sys.stderr.write

    sys.stdout.write = lambda text: writeWraper("stdout", stdout_write, text)
    sys.stderr.write = lambda text: writeWraper("stderr", stderr_write, text)


def overloads():
    overload_to_widgets()
    __overload_stdio()

    # TODO: remove overloads from sdk and just put the code there
    __overload_abstra_dashes_sdk()
    __overload_abstra_hooks_sdk()
    __overload_abstra_forms_sdk()



from ..widget_base import Input
from typing import List


class AnswerSheetInput(Input):
    type = "answer-sheet-input"

    def __init__(
        self, key: str, label: str, options: List, number_of_questions: int, **kwargs
    ):
        super().__init__(key)
        self.label = label
        self.options = options
        self.number_of_questions = number_of_questions
        self.required = kwargs.get("required", True)
        self.hint = kwargs.get("hint", None)
        self.columns = kwargs.get("columns", 1)
        self.full_width = kwargs.get("full_width", False)
        self.disabled = kwargs.get("disabled", False)

    def json(self, **kwargs):
        return {
            "type": self.type,
            "key": self.key,
            "label": self.label,
            "options": self.options,
            "numberOfQuestions": self.number_of_questions,
            "required": self.required,
            "hint": self.hint,
            "columns": self.columns,
            "fullWidth": self.full_width,
            "disabled": self.disabled,
        }

    def convert_answer(self, answer: str) -> str:
        return answer



import typing, traceback
from .common import (  # exported
    Message,
    StdioMessage,
    AuthValidJWTMessage,
    AuthInvalidJWTMessage,
    AuthRequireInfoMessage,
)
from ..utils import prepate_traceback


class ExecutionIdMessage(Message):
    type = "execution-id"

    def __init__(self, executionId: str):
        super().__init__({"executionId": executionId})


class ErrorMessage(Message):
    type = "error"

    def __init__(self, error: str):
        super().__init__({"error": error})


class ProgramStartFailedMessage(Message):
    type = "program-start-failed"

    def __init__(self, error: str, tb: traceback.StackSummary):
        super().__init__({"error": error, "stack": prepate_traceback(tb)})

    def to_json(self, is_preview: bool):
        if is_preview:
            return super().to_json(is_preview)
        return {"type": self.type}


class EvalReturnMessage(Message):
    type = "eval-return"

    def __init__(self, repr: str):
        super().__init__({"repr": repr})


class EvalErrorMessage(Message):
    type = "eval-error"

    def __init__(self, repr: str):
        super().__init__({"repr": repr})


class AutocompleteSuggestionsMessage(Message):
    type = "autocomplete:suggestions"

    def __init__(self, suggestionsFor: str, suggestions: typing.List[str]):
        super().__init__({"suggestions": suggestions, "suggestionsFor": suggestionsFor})


class WidgetsComputedMessage(Message):
    type = "widgets-computed"

    def __init__(
        self,
        props: dict,
        variables: dict,
        errors: dict,
        stateTimestamp: typing.Union[int, str],
    ):
        super().__init__(
            {
                "props": props,
                "variables": variables,
                "errors": errors,
                "stateTimestamp": stateTimestamp,
            }
        )


class WidgetsComputedGeneralErrorMessage(Message):
    type = "widgets-computed"

    def __init__(
        self,
        repr: str,
    ):
        super().__init__({"errors": {"general": {"repr": repr}}})


class RedirectMessage(Message):
    type = "redirect"

    def __init__(self, url: str, query_params: dict):
        super().__init__({"url": url, "queryParams": query_params})


class AlertMessage(Message):
    type = "alert"

    def __init__(self, message: str, severity: str):
        super().__init__({"message": message, "severity": severity})


class ExecuteJSRequestMessage(Message):
    type = "execute-js:request"

    def __init__(self, code: str, context: dict):
        super().__init__({"code": code, "context": context})
